#ifndef OBJ07_DB_API_H
#define OBJ07_DB_API_H

/* RDM 11.0.1.551 */

#include "Db.h"
#include "Cursor.h"
#include "obj07_db.h"

namespace RDM_CPP {

class Cursor_student;
class Cursor_course;
class Cursor_course_registration;

/*! \brief The obj07_db Db class
 *
 * This class provides the implementation for the obj07_db database
*/
class Db_obj07_db : public Db {
public:
    /*! \brief Open the obj07_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj07_db object
     */
    static Db_obj07_db Open (char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj07_db database using the default name
     *
     * \param mode The open mode to use
     * \return A validated Db_obj07_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj07_db Open (wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    /*! \brief Open the obj07_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj07_db object
     */
    static Db_obj07_db Open (const char *name, const char *mode="s") throw (rdm_exception&);

    /*! \brief Open the obj07_db database using the specified name
     *
     * \param name The database name to use for the open call
     * \param mode The open mode to use
     * \return A validated Db_obj07_db object
     */
#if !defined(RDM_NO_UNICODE)
    static Db_obj07_db Open (const wchar_t *name, const wchar_t *mode=L"s") throw (rdm_exception&);
#endif /*RDM_NO_UNICODE*/

    //! Db_obj07_db Default Constructor
    Db_obj07_db (void) throw ();
    /*! \brief Db_obj07_db Constructor
     *
     * \param db A Db object used to create the Db_obj07_db
     */
    Db_obj07_db (const Db& db) throw ();
    //! Db_obj07_db Copy Constructor
    Db_obj07_db (const Db_obj07_db&) throw ();

    //! Db_obj07_db assignment operator
    Db_obj07_db& operator = (const Db&) throw (rdm_exception&);

    //! Db_obj07_db assignment operator
    Db_obj07_db& operator = (const Db_obj07_db&) throw ();

    /*! \brief Db_obj07_db destructor
     *
     * Remove a reference to the underlying database.  The database is closed when the last reference is gone.
     */
    ~Db_obj07_db ();

    /*! \brief Create a new student record
     *
     * \returns A singleton Cursor_student object with default field values
     *
     * Create a new student record using default values
     */
    Cursor_student New_student_record (void) const throw (rdm_exception&);

    /*! \brief Create a new student record
     *
     * \param fields The field values for the new student record
     * \returns A singleton Cursor_student object with the specified field values
     *
     * Create a new student record using the specified values
     */
    Cursor_student New_student_recordWithFieldValues (struct student *fields) const throw (rdm_exception&);

    /*! \brief Create a new course record
     *
     * \returns A singleton Cursor_course object with default field values
     *
     * Create a new course record using default values
     */
    Cursor_course New_course_record (void) const throw (rdm_exception&);

    /*! \brief Create a new course record
     *
     * \param fields The field values for the new course record
     * \returns A singleton Cursor_course object with the specified field values
     *
     * Create a new course record using the specified values
     */
    Cursor_course New_course_recordWithFieldValues (struct course *fields) const throw (rdm_exception&);

    /*! \brief Create a new course_registration record
     *
     * \returns A singleton Cursor_course_registration object with default field values
     *
     * Create a new course_registration record using default values
     */
    Cursor_course_registration New_course_registration_record (void) const throw (rdm_exception&);

    /*! \brief Create a new course_registration record
     *
     * \param fields The field values for the new course_registration record
     * \returns A singleton Cursor_course_registration object with the specified field values
     *
     * Create a new course_registration record using the specified values
     */
    Cursor_course_registration New_course_registration_recordWithFieldValues (struct course_registration *fields) const throw (rdm_exception&);

    /*! \brief Obtain student cursor position at the first student record
     * \returns student cursor positioned at the first student record
     *
     * Create a new student cursor for doing a table scan of the student records.
     * The cursor is positioned at the first student record
     */
    Cursor_student Get_student_records (void) const throw (rdm_exception&);

    /*! \brief Obtain course cursor position at the first course record
     * \returns course cursor positioned at the first course record
     *
     * Create a new course cursor for doing a table scan of the course records.
     * The cursor is positioned at the first course record
     */
    Cursor_course Get_course_records (void) const throw (rdm_exception&);

    /*! \brief Obtain course_registration cursor position at the first course_registration record
     * \returns course_registration cursor positioned at the first course_registration record
     *
     * Create a new course_registration cursor for doing a table scan of the course_registration records.
     * The cursor is positioned at the first course_registration record
     */
    Cursor_course_registration Get_course_registration_records (void) const throw (rdm_exception&);

    /*! \brief Obtain a student cursor based on the name key
     * \returns student cursor positioned at the first student name key value
     *
     * Create a new student cursor based on the name key and position the cursor
     * at the first record
     */
    Cursor_student Get_student_recordsBy_name () const throw (rdm_exception&);

    /*! \brief Obtain a course cursor based on the id key
     * \returns course cursor positioned at the first course id key value
     *
     * Create a new course cursor based on the id key and position the cursor
     * at the first record
     */
    Cursor_course Get_course_recordsBy_id () const throw (rdm_exception&);

private:
    /*! \brief Db_obj07_db reference operator */
    Db_obj07_db* operator & (void);
    /*! \brief Db_obj07_db constructor */
    Db_obj07_db (IDb *idb);
    /*! \brief Get the Db_obj07_db dbType
     * \return The Db_obj07_db dbType
     */
    static dbType Type (void);
};

/*! \brief The student Cursor class
 *
 * This class provides the implementation for the student Cursor
*/
class Cursor_student : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_student objects
     */
    static recordType Type (void);

    /*! Cursor_student Default Constructor */
    Cursor_student (void) throw ();
    /*! \brief Cursor_student Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_student object
     */
    Cursor_student (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_student Copy Constructor */
    Cursor_student (const Cursor_student&) throw ();

    /*! \brief Cursor_student assignment operator */
    Cursor_student& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_student assignment operator */
    Cursor_student& operator = (const Cursor_student& cursor) throw ();

    /*! \brief Create a new course_registration cursor for attends set navigation from this record
     * \return course_registration cursor contain the member records related to this student
     */
    Cursor_course_registration Get_attends_members (void) const throw (rdm_exception&);

    /*! \brief Get the member count for the attends set
     * \return The number of member records related to this course_registration owner throught the student set
     */
    int32_t Get_attends_count (void) const throw (rdm_exception&);

    /*! \brief Create a new student cursor doing name key navigation.
     *
     * \return student cursor ordered by the name key
     *
     *  Create a new student cursor using name key navigation and position the
     *  new cursor at our current position
     */
    Cursor_student GetRecordsBy_name () const throw (rdm_exception&);

    /*! \brief Position the student cursor to the name key value
     *
     * \param name The name value to position the cursor at
     *
     */
    void KeyFindBy_name (char name_val[100]) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct student *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct student *fields) const throw (rdm_exception&);

    /*! \brief Set the name field value for the current record
     * \param name The field value that will be written to the record at the current cursor position
     */
    void Set_name (const char name_val[100]) const throw (rdm_exception&);

    /*! \brief Get the name field value for the current record
     * \param name A pointer where the field value from the current cursor position will be written
     */
    void Get_name (char name_val[100]) const throw (rdm_exception&);

    /*! \brief Cursor_student Destructor
     *
     * Release the cursor
     */
    ~Cursor_student ();

private:
    friend class Db_obj07_db;
    friend class Cursor_course;
    friend class Cursor_course_registration;
};

/*! \brief The course Cursor class
 *
 * This class provides the implementation for the course Cursor
*/
class Cursor_course : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_course objects
     */
    static recordType Type (void);

    /*! Cursor_course Default Constructor */
    Cursor_course (void) throw ();
    /*! \brief Cursor_course Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_course object
     */
    Cursor_course (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_course Copy Constructor */
    Cursor_course (const Cursor_course&) throw ();

    /*! \brief Cursor_course assignment operator */
    Cursor_course& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_course assignment operator */
    Cursor_course& operator = (const Cursor_course& cursor) throw ();

    /*! \brief Create a new course_registration cursor for attended_by set navigation from this record
     * \return course_registration cursor contain the member records related to this course
     */
    Cursor_course_registration Get_attended_by_members (void) const throw (rdm_exception&);

    /*! \brief Get the member count for the attended_by set
     * \return The number of member records related to this course_registration owner throught the course set
     */
    int32_t Get_attended_by_count (void) const throw (rdm_exception&);

    /*! \brief Create a new course cursor doing id key navigation.
     *
     * \return course cursor ordered by the id key
     *
     *  Create a new course cursor using id key navigation and position the
     *  new cursor at our current position
     */
    Cursor_course GetRecordsBy_id () const throw (rdm_exception&);

    /*! \brief Position the course cursor to the id key value
     *
     * \param id The id value to position the cursor at
     *
     */
    void KeyFindBy_id (char id_val[10]) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct course *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct course *fields) const throw (rdm_exception&);

    /*! \brief Set the id field value for the current record
     * \param id The field value that will be written to the record at the current cursor position
     */
    void Set_id (const char id_val[10]) const throw (rdm_exception&);

    /*! \brief Get the id field value for the current record
     * \param id A pointer where the field value from the current cursor position will be written
     */
    void Get_id (char id_val[10]) const throw (rdm_exception&);

    /*! \brief Set the title field value for the current record
     * \param title The field value that will be written to the record at the current cursor position
     */
    void Set_title (const char title_val[100]) const throw (rdm_exception&);

    /*! \brief Get the title field value for the current record
     * \param title A pointer where the field value from the current cursor position will be written
     */
    void Get_title (char title_val[100]) const throw (rdm_exception&);

    /*! \brief Cursor_course Destructor
     *
     * Release the cursor
     */
    ~Cursor_course ();

private:
    friend class Db_obj07_db;
    friend class Cursor_student;
    friend class Cursor_course_registration;
};

/*! \brief The course_registration Cursor class
 *
 * This class provides the implementation for the course_registration Cursor
*/
class Cursor_course_registration : public Cursor {
public:
    /*! \brief Get the type for objects of this class
     * \return The type for Cursor_course_registration objects
     */
    static recordType Type (void);

    /*! Cursor_course_registration Default Constructor */
    Cursor_course_registration (void) throw ();
    /*! \brief Cursor_course_registration Constructor
     *
     * \param cursor A Cursor object used to create the Cursor_course_registration object
     */
    Cursor_course_registration (const Cursor& cursor) throw (rdm_exception&);
    /*! \brief Cursor_course_registration Copy Constructor */
    Cursor_course_registration (const Cursor_course_registration&) throw ();

    /*! \brief Cursor_course_registration assignment operator */
    Cursor_course_registration& operator = (const Cursor& cursor) throw (rdm_exception&);

    /*! \brief Cursor_course_registration assignment operator */
    Cursor_course_registration& operator = (const Cursor_course_registration& cursor) throw ();

    /*! \brief Connect the record at the current cursor position to the owner cursor's attends set chain
     * \param owner student collection whose current position is the set owner we add the member to
     *
     * The current cursor must not be connected to an owner
     */
    void Connect_attends (const Cursor_student &owner) const throw (rdm_exception&);

    /*! \brief Connect the record at the current cursor position to the owner cursor's attends set chain
     * \param owner student collection whose current position is the set owner we add the member to
     *
     * The function will disconnect the current cursor from an it's existing owner before connecting it to the new owner
     */
    void Reconnect_attends (const Cursor_student &owner) const throw (rdm_exception&);

    /*! \brief Disconnect the record at the current cursor position from the attends set
     */
    void Disconnect_attends (void) const throw (rdm_exception&);

    /*! \brief Create a new Singleton student containing the attends owner record
     * \return A Singleton student containing the attends owner record
     */
    Cursor_student Get_attends_owner (void) const throw (rdm_exception&);

    /*! \brief Create a new course_registration cursor contain records with the same attends owner as the record at the current cursor position
     * \return A course_registration cursor contain the member records related to the same student owner as the current cursor
     */
    Cursor_course_registration Get_attends_siblings (void) const throw (rdm_exception&);

    /*! \brief Connect the record at the current cursor position to the owner cursor's attended_by set chain
     * \param owner course collection whose current position is the set owner we add the member to
     *
     * The current cursor must not be connected to an owner
     */
    void Connect_attended_by (const Cursor_course &owner) const throw (rdm_exception&);

    /*! \brief Connect the record at the current cursor position to the owner cursor's attended_by set chain
     * \param owner course collection whose current position is the set owner we add the member to
     *
     * The function will disconnect the current cursor from an it's existing owner before connecting it to the new owner
     */
    void Reconnect_attended_by (const Cursor_course &owner) const throw (rdm_exception&);

    /*! \brief Disconnect the record at the current cursor position from the attended_by set
     */
    void Disconnect_attended_by (void) const throw (rdm_exception&);

    /*! \brief Create a new Singleton course containing the attended_by owner record
     * \return A Singleton course containing the attended_by owner record
     */
    Cursor_course Get_attended_by_owner (void) const throw (rdm_exception&);

    /*! \brief Create a new course_registration cursor contain records with the same attended_by owner as the record at the current cursor position
     * \return A course_registration cursor contain the member records related to the same course owner as the current cursor
     */
    Cursor_course_registration Get_attended_by_siblings (void) const throw (rdm_exception&);

    /*! \brief Set all of the field values of the record at the current cursor position
     * \param fields A const pointer with the field values to be written to the current cursor position
     */
    void SetFieldValues (const struct course_registration *fields) const throw (rdm_exception&);

    /*! \brief Get all of the field values for the record at the current cursor position
     * \param fields A pointer where the record values from the current cursor position will be written
     */
    void GetFieldValues (struct course_registration *fields) const throw (rdm_exception&);

    /*! \brief Cursor_course_registration Destructor
     *
     * Release the cursor
     */
    ~Cursor_course_registration ();

private:
    friend class Db_obj07_db;
    friend class Cursor_student;
    friend class Cursor_course;
};

}
#endif    /* OBJ07_DB_API_H */
